import nude
from nude import Nude


def get(image_path):
    return get_nude(image_path)

def get_nude(image_path):

    """
    schema:isFamilyFriendly = False
    """

    return nude.is_nude(image_path)




import pytesseract
import os


#os.system("install-pkg tesseract-ocr")
#os.system("python -m pytest tests -vv")

try:
    test = pytesseract.pytesseract.get_tesseract_version()
except:
    os.system("install-pkg tesseract-ocr")



"""
Requires installing tesseract-ocr:
install-pkg tesseract-ocr

"""
pytesseract.pytesseract.tesseract_cmd = "tesseract"

os.environ["TESSDATA_PREFIX"] = "/home/runner/.apt/usr/share/tesseract-ocr/4.00/tessdata/"



def get(image_object):

    return _get_text(image_object)


def _get_text(image_object):
    """Does OCR for image file
    """

    img = image_object.get('temp:pil_img', None)

    text = pytesseract.image_to_string(img)

    output_record = {}
    output_record['@id'] = image_object.get('@id', None)
    output_record['@type'] = image_object.get('@type', None)

    output_record['schema:embeddedTextCaption'] = text



    return [output_record]



import requests
import cv2
import numpy as np
import requests
from PIL import Image
from io import BytesIO
from urllib.request import Request, urlopen  # Python 3
import pyimgbox
import aiohttp

def get(record):

    return get_pil_img(record)


def get_pil_img(record):
    
    filepath = record.get('temp:filepath', None)
    pil_img = record.get('temp:pil_img', None)
    cv2_img = record.get('temp:cv2_img', None)
    url = record.get('schema:contentUrl', None)

    if pil_img:
        return pil_img
    elif filepath:
        return get_image_from_path(filepath)
    elif url:
        return get_image_from_url(url)
    
    

def get_cv2_img(pil_img):
    return get_cv2_from_pil(pil_img)



def get_image_from_path(image_object, filepath):
    pil_img = Image.open(filepath) 

    return pil_img


def get_image_from_url(url):
    
    # Load image
    headers = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36',
    'referer': 'https://youramateurporn.com/'
    }

    try:
        r = requests.get(url, headers = headers, timeout = 20)
        pil_img = Image.open(BytesIO(r.content))

        return pil_img

    except:
        return None

async def get_image_from_url_async(url):
    
    # Load image
    headers = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36',
    'referer': 'https://youramateurporn.com/'
    }

    timeout = aiohttp.ClientTimeout(total=10)

    try:
        async with aiohttp.ClientSession(headers=headers, timeout=timeout) as session:
            async with session.get(url) as response:
                content = await response.read()

        pil_img = Image.open(BytesIO(content))

        return pil_img

    except Exception as e:
        print(e)
        return None

def get_cv2_from_pil(pil_img):

    if type(pil_img) is bool: 
        pil_img = pil_img.astype(np.uint8) * 255

    cv2_img = cv2.cvtColor(np.array(pil_img), cv2.COLOR_RGB2BGR)

    return cv2_img






import cv2
import uuid

from kraken_image_processing.extractors import kraken_image_face_gender 
from kraken_image_processing.extractors import kraken_image_face_age
from kraken_image_processing.extractors import kraken_image_face_emotions 
from kraken_image_processing.extractors import kraken_image_face_embeddings 
import pkg_resources


#model = pkg_resources.resource_stream(__name__, 'models/nn4.small2.v1.t7')

model = 'models/nn4.small2.v1.t7'

embedder = cv2.dnn.readNetFromTorch(model)



def get(image_object):
    return process_faces(image_object)


def process_faces(record):

    faces = get_faces(record)

    records = []

    new_image_object = {
        '@type': record.get('@type', None),
        '@id': record.get('@id', None)
    }

    for f in faces:

        record_type = f.get('@type', None)
        record_id = f.get('@id', None)
        img = f.get('temp:cv2_img', None)

        width, height, colors = img.shape
        if not width or width < 20 or not height or height < 30:
            continue

        record_type = 'schema:person'
        record_id = str(uuid.uuid4())


        # get emotion
        try:
            i = {'@type': record_type, '@id': record_id}
            i['emotion'] = kraken_image_face_emotions.get(img)
            i['datasource'] = 'kraken_image_face_emotion'
            records.append(i)
        except:
            print('error emotions')

        # get gender
        try:
            i = {'@type': record_type, '@id': record_id}
            i['schema:gender'] = kraken_image_face_gender.get(img)
            i['datasource'] = 'kraken_image_face_gender'
            records.append(i)
        except:
            print('error gender')

        # get age
        try:
            i = {'@type': record_type, '@id': record_id}
            i['schema:age'] = kraken_image_face_age.get(img)
            i['datasource'] = 'kraken_image_face_age'
            records.append(i)
        except:
            print('error age')


        # get embeddings
        try:
            i = {'@type': record_type, '@id': record_id}
            i['embeddings'] = kraken_image_face_embeddings.get(img)
            i['datasource'] = 'kraken_image_face_embeddings'
            records.append(i)
        except:
            print('error embeddings')


        new_image_object['schema:about'] = records

    return [new_image_object]



def get_faces(record):
    # returns cv2 faces


    img = record.get('temp:cv2_img', None)


    # Load the cascade
    face_cascade = cv2.CascadeClassifier('kraken_image_processing/models/haarcascade_frontalface_default.xml')
    
    # Convert into grayscale
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    # Detect faces
    faces = face_cascade.detectMultiScale(gray, 1.1, 4)
    # Draw rectangle around the faces

    persons = []
    for (x, y, w, h) in faces:

        record_type = 'schema:person'
        record_id = str(uuid.uuid4())
        person = {'@type': record_type, '@id': record_id}


        crop_img = img[y:y+h, x:x+w]

        filename = 'images/faces/' + record_id + '.png'
        cv2.imwrite(filename, crop_img)
        person['temp:cv2_img'] = crop_img
        persons.append(person)

    return persons







'''
from imageai.Detection import ObjectDetection
import os


def get(image_object):
    return get_objects(image_object)


def get_objects(image_object):
    """Retrieves objects from image file
    """
    img = image_object.get('temp:pil_img', None)

    execution_path = os.getcwd()
    print(execution_path)

    detector = ObjectDetection()
    detector.setModelTypeAsYOLOv3()
    #detector.setModelPath( os.path.join(execution_path , "yolo.h5"))
    detector.setModelPath( "yolo.h5")

    detector.loadModel()
    detections = detector.detectObjectsFromImage(input_image=img,  minimum_percentage_probability=30)

    for eachObject in detections:
        print(eachObject["name"] , " : ", eachObject["percentage_probability"], " : ", eachObject["box_points"] )
        print("--------------------------------")
    return detections


'''


import cv2
import uuid
from fer import FER
import pkg_resources


#model = pkg_resources.resource_stream(__name__, 'models/nn4.small2.v1.t7')
model =  'models/nn4.small2.v1.t7'

embedder = cv2.dnn.readNetFromTorch(model)


def get(image):
    return get_embeddings(image)





def get_embeddings(image):
    """
    https://github.com/Ravi-Singh88/Face-Recognition-OpenCV-Facenet/blob/master/extract_embeddings.py
    
    """
    faceBlob = cv2.dnn.blobFromImage(image, 1.0 / 255,
				(96, 96), (0, 0, 0), swapRB=True, crop=False)
    embedder.setInput(faceBlob)
    vec = embedder.forward()
    embeddings = vec.flatten()
    return embeddings



import os
from flask import Flask
from flask import request
from flask import Response
from flask import redirect
from flask import url_for
from flask import jsonify
import requests
import json
import datetime
import time
import uuid
import random
from kraken_image_processing import kraken_image_processing as k   


# Initalize app
test_mode = False


# Initialize flask app
app = Flask(__name__,
            static_url_path='',
            static_folder='static',
            template_folder='templates')
app.secret_key = b'_5#mn"F4Q8z\n\xec]/'
app.config['SEND_FILE_MAX_AGE_DEFAULT'] = 0



@app.route('/', methods=['GET'])
def main_get():

    url = 'https://imageproc2.tactik8.repl.co/api'
    files = {'file': open('img.png', 'rb')}

    params = {'@type':'schema:imageObject', '@id': 'some_id'}
    #data = json.dumps(record, default = str)

    #r = requests.post(url, params=params, files=files)
    #print(r.json())

    return Response('ok')


@app.route('/api', methods=['GET'])
def api_get():
    """Process get data
    """

    return Response('ok')


@app.route('/api', methods=['POST'])
def api_post():
    """Process get data
    """
    
    file = request.files['file'].read()
    record = {
        '@id': request.values.get('@id'),
        '@id': request.values.get('@type')
        }

    records = k.process(record, file)
    print(records)
    return Response('ok')

    # save the single "profile" file
    uploads_dir = os.path.join(app.instance_path, 'uploads')
    uploads_dir = 'uploads'


    image = request.files['file']
    image.save(os.path.join(uploads_dir, image.filename))

    return jsonify('ok')





def run_api():
    app.run(host='0.0.0.0', debug=False)




import requests
import os

def download_models():

    print('Models download started.')


    directory = 'models'

    create_directory(directory)
    models = []
    models.append(('nn4.small2.v1.t7', 'https://storage.cmusatyalab.org/openface-models/nn4.small2.v1.t7')) 
    models.append(('age_net.caffemodel', 'https://github.com/GilLevi/AgeGenderDeepLearning/raw/master/models/age_net.caffemodel'))

    models.append(('age_net.caffemodel', 'https://github.com/GilLevi/AgeGenderDeepLearning/raw/master/models/age_net.caffemodel'))

    models.append(('gender_net.caffemodel', 'https://github.com/eveningglow/age-and-gender-classification/raw/master/model/gender_net.caffemodel'))

    models.append(('haarcascade_frontalface_default.xml', 'https://github.com/kipr/opencv/raw/master/data/haarcascades/haarcascade_frontalface_default.xml'))

    models.append(('age_deploy.prototxt', 'https://raw.githubusercontent.com/spmallick/learnopencv/master/AgeGender/age_deploy.prototxt'))

    models.append(('gender_deploy.prototxt', 'https://raw.githubusercontent.com/spmallick/learnopencv/master/AgeGender/gender_deploy.prototxt'))



    for file, url in models:
        filepath = directory + '/' + file
        if not check_if_exist(filepath):
            download_file(url, filepath)

    print('Models download completed.')

def create_directory(directory):
    try:
        os.makedirs(directory)
    except OSError as e:
        a=1



def check_if_exist(filepath):

    return os.path.isfile(filepath)


def download_file(url, filepath):
    print('Downloading ', filepath)
    r = requests.get(url)
    open(filepath, 'wb').write(r.content)


download_models()



from kraken_image_processing.extractors import kraken_image_hash 
from kraken_image_processing.extractors import kraken_image_nude 




def get(record):
    return process_image(record)


def process_image(record):

    records = []

    cv2_img = record.get('temp:cv2_img', None)
    pil_img = record.get('temp:pil_img', None)


    record_type = record.get('@type', None)
    record_id = record.get('@id', None)


    # Get size
    i = {'@type': record_type, '@id': record_id}
    i['schema:height'], i['schema:width'], colors = cv2_img.shape
    i['datasource'] = 'kraken_image_generic'
    records.append(i)

    # get image hash
    i = {'@type': record_type, '@id': record_id}
    i['schema:hash'] = kraken_image_hash.get(cv2_img)
    i['datasource'] = 'kraken_image_generic'

    records.append(i)

    # get image hash256
    i = {'@type': record_type, '@id': record_id}
    i['schema:sha256'] = kraken_image_hash.get_sha256(pil_img)
    i['datasource'] = 'kraken_image_generic'

    records.append(i)


    # get image rating
    i = {'@type': record_type, '@id': record_id}
    i['schema:isFamilyFriendly'] = not kraken_image_nude.get(pil_img)
    i['datasource'] = 'kraken_image_nude'

    records.append(i)


    return records



import asyncio
import aiohttp
import pyimgbox




def post(input_records = None):
    """Takes schema record entity and load image from path defined at temp:filepath
    Returns new imageobject with contenturl and thumbnailurl
    """

    #loop = asyncio.get_event_loop()
    loop = asyncio.new_event_loop()
    input_record = {
        '@id': 'test_id', 
        '@type': 'schema:ImageObject', 
        'temp:filepath': 'image.jpg'
    }

    output_records = loop.run_until_complete(post_async(input_record))

    return output_records

async def post_async(input_records):
    """Takes schema record entity and load image from path defined at temp:filepath
    Returns new imageobject with contenturl and thumbnailurl
    """

    if not isinstance(input_records, list): 
        input_records = [input_records]

    # Write files to site
    tasks = []
    tasks.append(asyncio.ensure_future(post_to_image_sharing(input_records)))
    output_records = await asyncio.gather(*tasks, return_exceptions=True)
    
    # Convert back to non list
    while type(output_records) is list and len(output_records) <= 1:
        output_records = output_records[0]

    return output_records


async def post_to_image_sharing(input_records):


    # Get filepaths from records
    filepaths = []
    for r in input_records:
        filepaths.append(r.get('temp:filepath', None))

    # Load filepaths
    async with pyimgbox.Gallery(title="References", adult = True, square_thumbs = True, thumb_width = 200) as gallery:
        output_records = []
        try:
            async for submission in gallery.add(filepaths):

                record_imageObject = {
                    '@type': 'schema:ImageObject',
                    'schema:thumbnailUrl': submission.thumbnail_url,
                    'schema:contentUrl': submission.image_url
                }
                
                # Find corresponding record id
                for i in input_records:
                    if i.get('temp:filepath', None) == submission.filepath:
                        record_imageObject['@id'] = i.get('@id', None)
                
                output_records.append(record_imageObject)
        except Exception as e:
            print('error', e)

    # Match files uploaded with original record
   

    return output_records

post()





import cv2
import numpy as np
import hashlib
import io

"""
API
"""





def get(image_path):
    """Given a path name, return the hash of the image
    """
    try:
        return get_phash(image_path)
    except:
        return None

"""
Helpers
"""

def get_sha256(img):
    m = hashlib.md5()
    with io.BytesIO() as memf:
        img.save(memf, 'PNG')
        data = memf.getvalue()
        m.update(data)
    return m.hexdigest()


def get_phash(image_path):
    """Given a path name, return the perceptual hash of the image
    """
    img = image_path


    # resize image and convert to gray scale
    img = cv2.resize(img, (64, 64))
    img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    img = np.array(img, dtype = np.float32)
    # calculate dct of image 
    dct = cv2.dct(img)
    # to reduce hash length take only 8*8 top-left block 
    # as this block has more information than the rest
    dct_block = dct[: 8, : 8]
    # caclulate mean of dct block excluding first term i.e, dct(0, 0)
    dct_average = (dct_block.mean() * dct_block.size - dct_block[0, 0]) / (dct_block.size - 1)
    # convert dct block to binary values based on dct_average
    dct_block[dct_block < dct_average] = 0.0
    dct_block[dct_block != 0] = 1.0
    # store hash value
    hash_value = hash_array_to_hash_hex(dct_block.flatten())

    return hash_value


def hash_array_to_hash_hex(hash_array):
  # convert hash array of 0 or 1 to hash string in hex
  hash_array = np.array(hash_array, dtype = np.uint8)
  hash_str = ''.join(str(i) for i in 1 * hash_array.flatten())
  return (hex(int(hash_str, 2)))

def hash_hex_to_hash_array(hash_hex):
  # convert hash string in hex to hash values of 0 or 1
  hash_str = int(hash_hex, 16)
  array_str = bin(hash_str)[2:]
  return np.array([i for i in array_str], dtype = np.float32)






import cv2
import math
import pkg_resources



genderProto = "models/gender_deploy.prototxt"
genderModel = "models/gender_net.caffemodel"


#genderProto = pkg_resources.resource_stream(__name__, 'models/gender_deploy.prototxt')

#genderModel = pkg_resources.resource_stream(__name__, 'models/gender_net.caffemodel')



MODEL_MEAN_VALUES = (78.4263377603, 87.7689143744, 114.895847746)

genderList = ['Male', 'Female']



#load the network
genderNet = cv2.dnn.readNet(genderModel, genderProto)




def get(image_path):
    return get_gender(image_path)

def get_gender(image_path):

    # Load image
    if isinstance(image_path, str):
        image = cv2.imread(image_path)
    else:
        image = image_path



    blob = cv2.dnn.blobFromImage(image, 1.0, (227, 227), MODEL_MEAN_VALUES, swapRB=False)
    genderNet.setInput(blob)
    genderPreds = genderNet.forward()
    gender = genderList[genderPreds[0].argmax()]



    return gender


from kraken_image_processing import download_models

download_models.download_models()

from kraken_image_processing import kraken_image_io
from kraken_image_processing.extractors import kraken_image_generic
from kraken_image_processing.extractors import kraken_image_person
from kraken_image_processing.extractors import kraken_image_face
from kraken_image_processing.extractors import kraken_image_text

from kraken_image_processing import download_models
import os


from io import BytesIO
#from kraken_image_processing.extractors import kraken_image_objects


from kraken_image_processing import post_to_image_share
from PIL import Image



os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'

def get(image_file):
    return process(image_file)



async def load_async(record):
    """Load image from url
    """
    url = record.get('schema:contentUrl', None)
    record['temp:pil_img'] = await kraken_image_io.get_image_from_url_async(url)

    return record


def process(record, image_byte = None):
    """Process record. If file_path provided, load image from disk instead of url.
    """

    records = []
    
    if image_byte:
        file_jpgdata = BytesIO(image_byte)
        pil_img = Image.open(file_jpgdata)

        record['temp:pil_img'] = pil_img


    # Load image
    pil_img = record.get('temp:pil_img', None)
    if not pil_img:
        pil_img = kraken_image_io.get_pil_img(record)
    record['temp:cv2_img'] = kraken_image_io.get_cv2_img(pil_img)

    # Process image generic
    print('Get generic')
    records += kraken_image_generic.get(record)

    # retrieve and process persons from image
    print('Get persons')
    records += kraken_image_person.get(record)

    # process faces from image
    print('Get faces')
    records += kraken_image_face.get(record)

    # process text from image
    print('Get text')
    records += kraken_image_text.get(record)

    # process objects from image
    print('Get text')
    #records += kraken_image_objects.get(record)

    # Save original image to share
    records += post_to_image_share.post(record)


    return records


async def process_async(record):
    """Process record. If file_path provided, load image from disk instead of url.
    """

    records = []
    
    # Load image
    pil_img = record.get('temp:pil_img', None)
    if not pil_img:
        pil_img = kraken_image_io.get_pil_img(record)
    record['temp:cv2_img'] = kraken_image_io.get_cv2_img(pil_img)

    # Process image generic
    print('Get generic')
    records += kraken_image_generic.get(record)

    # retrieve and process persons from image
    print('Get persons')
    records += kraken_image_person.get(record)

    # process faces from image
    print('Get faces')
    records += kraken_image_face.get(record)

    # process text from image
    print('Get text')
    records += kraken_image_text.get(record)



    # Save original image to share
    records += await post_to_image_share.post_async(record)


    return records






import cv2
import math
import pkg_resources

#ageProto = "kraken_image_processing/models/age_deploy.prototxt"
#ageModel = "kraken_image_processing/models/age_net.caffemodel"


#ageProto = pkg_resources.resource_stream(__name__, 'models/age_deploy.prototxt')

#ageModel = pkg_resources.resource_stream(__name__, 'models/age_net.caffemodel')

ageProto =  'models/age_deploy.prototxt'

ageModel = 'models/age_net.caffemodel'


MODEL_MEAN_VALUES = (78.4263377603, 87.7689143744, 114.895847746)
ageList = ['(0-2)', '(4-6)', '(8-12)', '(15-20)', '(25-32)', '(38-43)', '(48-53)', '(60-100)']


#load the network
ageNet = cv2.dnn.readNet(ageModel,ageProto)


def get(image_path):
    return get_age(image_path)

def get_age(image_path):

    # Load image
    if isinstance(image_path, str):
        image = cv2.imread(image_path)
    else:
        image = image_path



    blob = cv2.dnn.blobFromImage(image, 1.0, (227, 227), 
    MODEL_MEAN_VALUES, swapRB=False)
    
    ageNet.setInput(blob)
    agePreds = ageNet.forward()
    age = ageList[agePreds[0].argmax()]

    return age


import cv2
import uuid
from fer import FER



def get(image):
    return get_emotions(image)

def get_emotions(image):

    emo_detector = FER()
    # Capture all the emotions on the image
    captured_emotions = emo_detector.detect_emotions(image)
    top_emotion = emo_detector.top_emotion(image)

    e, p = top_emotion

    return e


import cv2
import uuid
import numpy as np



def get(image_path):
    #return get_persons(image_path)
    return []



def get_persons(image_path):
    """
    https://thedatafrog.com/en/articles/human-detection-video/
    """

    # Load image
    if isinstance(image_path, str):
        img = cv2.imread(image_path)
    else:
        img = image_path

    hog = cv2.HOGDescriptor()
    hog.setSVMDetector(cv2.HOGDescriptor_getDefaultPeopleDetector())

    # resizing for faster detection
    # using a greyscale picture, also for faster detection
    gray = cv2.cvtColor(img, cv2.COLOR_RGB2GRAY)

    # detect people in the image
    # returns the bounding boxes for the detected objects
    #boxes, weights = hog.detectMultiScale(image, winStride=(8,8) )
    boxes, weights = hog.detectMultiScale(img)

    boxes = np.array([[x, y, x + w, y + h] for (x, y, w, h) in boxes])

    cv2_persons = []
    for (xA, yA, xB, yB) in boxes:
        crop_img = img[yA:yB, xA:xB]
        cv2_persons.append(crop_img)
        # display the detected boxes in the colour picture
        cv2.rectangle(img, (xA, yA), (xB, yB),
                          (0, 255, 0), 2)
    filename = 'images_persons/' + str(uuid.uuid4()) + '.png'
    cv2.imwrite(filename, img)
    return cv2_persons

        


from kraken_image_processing import kraken_image_processing as k   

from kraken_class_entity.entity import Entity
from kraken_class_entity.entities import Entities
import asyncio
import uuid
import requests
import json
import datetime

def run():

    entities = Entities()
    entities.api_url = 'https://krakenengine.tactik8.repl.co/api'

    params = {'record_type': 'schema:Imageobject'}


    entities.search_api(params, 200, 0)
    loop = asyncio.get_event_loop()
    records = []
    for i in entities.entities:
        records.append(i.record)
    records = loop.run_until_complete(run_async(records))


    print(len(records))
    count = 0
    for i in records:
        img = i.get('temp:pil_img', None)
        if img:
            exist = True
            count += 1
        else:
            exist = False
        #print(i.get('@id', None), exist)
    print('Success:', count)


    # Process
    output_records = []
    actions = []
    count = 0
    for i in records:
        count += 1
        print('Processing record:', count)
        if i.get('temp:pil_img', None):
            

            start_time=datetime.datetime.now()

            result = k.get(i)
            output_records.append(result)
            end_time=datetime.datetime.now()
            action = {
                '@type': 'schema:action',
                '@id': str(uuid.uuid4()),
                'schema:object': i,
                'schema:result': result, 
                'schema:startTime': start_time,
                'schema:endTime': end_time,
                'schema:instrument': {'@type': 'schema:WebAPI', 'schema:url': 'https://replit.com/@tactik8/imageproc2#kraken_image_processing'},
                'datasource': 'image_proc.kraken_engine'
            }
            actions.append(action)
            post_action(action)
            #e = Entity()
            #e.load(action)
            #e.post_api()


    print('Output n: ', len(output_records))

    # Save records
    output_entities = Entities()
    #output_entities.load(actions)
    #output_entities.post_api()
    print('Saved')


def post_action(action):
    url = 'https://krakenengine.tactik8.repl.co/api'

    headers = {'Content-Type': 'application/json'}
    data = json.dumps(action, default = str)
    r = requests.post(url, headers = headers, data = data)



async def run_async(records):
    tasks = []
    for i in records:
        tasks.append(asyncio.ensure_future(k.load_async(i)))
    print('starting')
    records = await asyncio.gather(*tasks, return_exceptions=True)
    return records



import uuid

class ImageObject:
    def __init__(self):

        a=2

    
    def load(self, record):

        if isinstance(record, list) and len(record) == 1:
            record = record[0]

        if not record:
            return

        if not isinstance(record, dict):
            return None
            
        for k in record.keys():
            value = record.get(k, None)
            self.set(k, value)
        
        #self._normalize_type()
        #self._normalize_id()

        return True

    def dump(self):
        record = {}
        keys = [i for i in self.__dict__.keys() if i[:1] != '_']

        for k in keys:
            value = self.get(k)
            if type(value) is type(self):
                value = value.record
            record[k] = value
        return record

    def get(self, key):

        # Normalize key
        norm_key = key
        #norm_key = self._normalize_key(key)

        value = getattr(self, norm_key, None)
        #value = self._record.get(norm_key, None)

        return value


    def set(self, key, value):

        

        # Normalize key
        norm_key = key
        #norm_key = self._normalize_key(key)
        norm_value = value
        #norm_value = self._normalize_value(norm_key, value)
        setattr(self, norm_key, norm_value)

        return
